#
# quickml/utils - a part of quickml server
#
# Copyright (C) 2002 Satoru Takabayashi <satoru@namazu.org> 
#     All rights reserved.
#     This is free software with ABSOLUTELY NO WARRANTY.
#
# You can redistribute it and/or modify it under the terms of 
# the GNU General Public License version 2.
#
$KCODE = 'e'
require 'kconv'
require 'net/smtp'
require 'ftools'

class TooLongLine < Exception; end
class IO
  def safe_gets (max_length = 1024)
    s = ""
    until self.eof?
      c = self.read(1)
      s << c
      if s.length > max_length
	raise TooLongLine
      end
      if c == "\n"
	return s
      end
    end
    if s.empty? then nil else s end
  end
end

class String
  def xchomp!
    self.chomp!("\n")
    self.chomp!("\r")
  end

  def normalize_eol!
    self.xchomp!
    self << "\n"
  end

  def xchomp
    self.chomp("\n").chomp("\r")
  end
end

class TCPSocket
  def address
    peeraddr[3]
  end

  def hostname
    peeraddr[2]
  end
end

class DupFile
  def initialize (filename1, filename2, mode = "r")
    @write_p = if /[aw]/.match(mode) then true else false end
    @f2 = File.open(filename2, mode) if @write_p
    @f1 = File.open(filename1, mode)
  end

  def method_missing (symbol, *args)
    @f2.send(symbol, *args) if @write_p
    @f1.send(symbol, *args)
  end

  def close
    @f2.close if @write_p
    @f1.close
  end
end

class File
  @@backup_dir = nil
  def self.backup_dir= (dir)
    @@backup_dir = dir
  end

  def self.safe_open (filename, mode = "r")
    begin 
      f = if @@backup_dir and /[aw]/.match(mode)
	    backup_filename = File.join(@@backup_dir, filename)
	    File.mkpath(File.dirname(backup_filename))
	    DupFile.new(filename, backup_filename, mode)
	  else
	    File.open(filename, mode)
	  end
      if block_given?
	yield(f)
	f.close
      else
	return f
      end
    rescue => e
      STDERR.printf "%s: %s\n", $0, e.message
      exit(1)
    end
  end

  # To discard the method defined in ftools.rb
  class << self; remove_method :safe_unlink;  end
  def self.safe_unlink (filename)
    begin
      File.unlink(filename)
      if @@backup_dir
	backup_filename = File.join(@@backup_dir, filename)
	File.unlink(backup_filename)
      end
    rescue
    end
  end
end

class Integer
  # commify(12345) => "12,345"
  def commify
    numstr = self.to_s
    true while numstr.sub!(/^([-+]?\d+)(\d{3})/, '\1,\2')
    return numstr
  end
end
